@@ -7,6 +7,8 @@
     (NIL_P(str) ? warn_print(x) : (void)rb_str_cat_cstr(str, x))
 #define write_warn2(str, x, l) \
     (NIL_P(str) ? warn_print2(x, l) : (void)rb_str_cat(str, x, l))
+#define write_warn_enc(str, x, l, enc) \
+    (NIL_P(str) ? warn_print2(x, l) : (void)rb_enc_str_buf_cat(str, x, l, enc))
 #ifdef HAVE_BUILTIN___BUILTIN_CONSTANT_P
 #define warn_print(x) RB_GNUC_EXTENSION_BLOCK(	\
     (__builtin_constant_p(x)) ? 		\
@@ -103,7 +105,7 @@ print_errinfo(const VALUE eclass, const VALUE errat, const VALUE emesg, const VA
         if (highlight) write_warn(str, underline);
         write_warn(str, "unhandled exception");
         if (highlight) write_warn(str, reset);
-        write_warn2(str, "\n", 1);
+        write_warn(str, "\n");
     }
     else {
         VALUE epath;
@@ -123,17 +125,21 @@ print_errinfo(const VALUE eclass, const VALUE errat, const VALUE emesg, const VA
 }
 
 VALUE
-rb_decorate_message(const VALUE eclass, const VALUE emesg, int highlight)
+rb_decorate_message(const VALUE eclass, VALUE emesg, int highlight)
 {
     const char *einfo = "";
     long elen = 0;
+    rb_encoding *eenc;
 
-    VALUE str = rb_str_new2("");
+    VALUE str = rb_usascii_str_new_cstr("");
 
-    if (!NIL_P(emesg)) {
+    if (!NIL_P(emesg) && rb_enc_asciicompat(eenc = rb_enc_get(emesg))) {
         einfo = RSTRING_PTR(emesg);
         elen = RSTRING_LEN(emesg);
     }
+    else {
+        eenc = NULL;
+    }
     if (eclass == rb_eRuntimeError && elen == 0) {
         if (highlight) write_warn(str, underline);
         write_warn(str, "unhandled exception");
@@ -156,7 +162,7 @@ rb_decorate_message(const VALUE eclass, const VALUE emesg, int highlight)
             if (RSTRING_PTR(epath)[0] == '#')
                 epath = 0;
             if ((tail = memchr(einfo, '\n', elen)) != 0) {
-                write_warn2(str, einfo, tail - einfo);
+                write_warn_enc(str, einfo, tail - einfo, eenc);
                 tail++;		/* skip newline */
             }
             else {
@@ -170,23 +176,23 @@ rb_decorate_message(const VALUE eclass, const VALUE emesg, int highlight)
                     write_warn(str, reset);
                     write_warn(str, bold);
                 }
-                write_warn2(str, ")", 1);
+                write_warn(str, ")");
                 if (highlight) write_warn(str, reset);
             }
             if (tail && einfo+elen > tail) {
                 if (!highlight) {
-                    write_warn2(str, "\n", 1);
-                    write_warn2(str, tail, einfo+elen-tail);
+                    write_warn(str, "\n");
+                    write_warn_enc(str, tail, einfo+elen-tail, eenc);
                 }
                 else {
                     elen -= tail - einfo;
                     einfo = tail;
-                    write_warn2(str, "\n", 1);
+                    write_warn(str, "\n");
                     while (elen > 0) {
                         tail = memchr(einfo, '\n', elen);
                         if (!tail || tail > einfo) {
                             write_warn(str, bold);
-                            write_warn2(str, einfo, tail ? tail-einfo : elen);
+                            write_warn_enc(str, einfo, tail ? tail-einfo : elen, eenc);
                             write_warn(str, reset);
                             if (!tail) {
                                 break;
@@ -195,7 +201,7 @@ rb_decorate_message(const VALUE eclass, const VALUE emesg, int highlight)
                         elen -= tail - einfo;
                         einfo = tail;
                         do ++tail; while (tail < einfo+elen && *tail == '\n');
-                        write_warn2(str, einfo, tail-einfo);
+                        write_warn_enc(str, einfo, tail-einfo, eenc);
                         elen -= tail - einfo;
                         einfo = tail;
                     }
@@ -204,6 +210,8 @@ rb_decorate_message(const VALUE eclass, const VALUE emesg, int highlight)
         }
     }
 
+    RB_GC_GUARD(emesg);
+
     return str;
 }
 
