@@ -14,7 +14,7 @@ VALUE
 bug_s_fstring_fake_str(VALUE self)
 {
     static const char literal[] = "abcdefghijklmnopqrstuvwxyz";
-    struct RString fake_str;
+    struct RString fake_str = {RBASIC_INIT};
     return rb_fstring(rb_setup_fake_str(&fake_str, literal, sizeof(literal) - 1, 0));
 }
 
