@@ -53,8 +53,13 @@ function_memsize(const void *p)
 }
 
 const rb_data_type_t function_data_type = {
-    "fiddle/function",
-    {0, deallocate, function_memsize,},
+    .wrap_struct_name = "fiddle/function",
+    .function = {
+        .dmark = 0,
+        .dfree = deallocate,
+        .dsize = function_memsize
+    },
+    .flags = RUBY_TYPED_FREE_IMMEDIATELY | RUBY_TYPED_WB_PROTECTED,
 };
 
 static VALUE
@@ -375,10 +380,18 @@ function_call(int argc, VALUE argv[], VALUE self)
         (void)rb_thread_call_without_gvl(nogvl_ffi_call, &args, 0, 0);
     }
 
-    rb_funcall(mFiddle, rb_intern("last_error="), 1, INT2NUM(errno));
+    {
+        int errno_keep = errno;
 #if defined(_WIN32)
-    rb_funcall(mFiddle, rb_intern("win32_last_error="), 1, INT2NUM(errno));
+        DWORD error = WSAGetLastError();
+        int socket_error = WSAGetLastError();
+        rb_funcall(mFiddle, rb_intern("win32_last_error="), 1,
+                   ULONG2NUM(error));
+        rb_funcall(mFiddle, rb_intern("win32_last_socket_error="), 1,
+                   INT2NUM(socket_error));
 #endif
+        rb_funcall(mFiddle, rb_intern("last_error="), 1, INT2NUM(errno_keep));
+    }
 
     ALLOCV_END(alloc_buffer);
 
