@@ -1,11 +1,10 @@
 /*
- * $Id: ossl_digest.h 25189 2009-10-02 12:04:37Z akr $
  * 'OpenSSL for Ruby' project
  * Copyright (C) 2001-2002  Michal Rokos <m.rokos@sh.cvut.cz>
  * All rights reserved.
  */
 /*
- * This program is licenced under the same licence as Ruby.
+ * This program is licensed under the same licence as Ruby.
  * (See the file 'LICENCE'.)
  */
 #if !defined(_OSSL_DIGEST_H_)
@@ -14,9 +13,8 @@
 extern VALUE cDigest;
 extern VALUE eDigestError;
 
-const EVP_MD *GetDigestPtr(VALUE);
+const EVP_MD *ossl_evp_get_digestbyname(VALUE);
 VALUE ossl_digest_new(const EVP_MD *);
 void Init_ossl_digest(void);
 
 #endif /* _OSSL_DIGEST_H_ */
-
