@@ -7,12 +7,12 @@ RSpec.describe "gemcutter's dependency API" do
   it "should use the API" do
     gemfile <<-G
       source "#{source_uri}"
-      gem "rack"
+      gem "myrack"
     G
 
-    bundle :install, :artifice => "endpoint"
+    bundle :install, artifice: "endpoint"
     expect(out).to include("Fetching gem metadata from #{source_uri}")
-    expect(the_bundle).to include_gems "rack 1.0.0"
+    expect(the_bundle).to include_gems "myrack 1.0.0"
   end
 
   it "should URI encode gem names" do
@@ -21,7 +21,7 @@ RSpec.describe "gemcutter's dependency API" do
       gem " sinatra"
     G
 
-    bundle :install, :artifice => "endpoint", :raise_on_error => false
+    bundle :install, artifice: "endpoint", raise_on_error: false
     expect(err).to include("' sinatra' is not a valid gem name because it contains whitespace.")
   end
 
@@ -31,7 +31,7 @@ RSpec.describe "gemcutter's dependency API" do
       gem "rails"
     G
 
-    bundle :install, :artifice => "endpoint"
+    bundle :install, artifice: "endpoint"
     expect(out).to include("Fetching gem metadata from #{source_uri}/...")
     expect(the_bundle).to include_gems(
       "rails 2.3.2",
@@ -49,22 +49,22 @@ RSpec.describe "gemcutter's dependency API" do
       gem "net-sftp"
     G
 
-    bundle :install, :artifice => "endpoint"
+    bundle :install, artifice: "endpoint"
     expect(the_bundle).to include_gems "net-sftp 1.1.1"
   end
 
   it "should use the endpoint when using deployment mode" do
     gemfile <<-G
       source "#{source_uri}"
-      gem "rack"
+      gem "myrack"
     G
-    bundle :install, :artifice => "endpoint"
+    bundle :install, artifice: "endpoint"
 
     bundle "config set --local deployment true"
     bundle "config set --local path vendor/bundle"
-    bundle :install, :artifice => "endpoint"
+    bundle :install, artifice: "endpoint"
     expect(out).to include("Fetching gem metadata from #{source_uri}")
-    expect(the_bundle).to include_gems "rack 1.0.0"
+    expect(the_bundle).to include_gems "myrack 1.0.0"
   end
 
   it "handles git dependencies that are in rubygems" do
@@ -75,12 +75,12 @@ RSpec.describe "gemcutter's dependency API" do
 
     gemfile <<-G
       source "#{source_uri}"
-      git "#{file_uri_for(lib_path("foo-1.0"))}" do
+      git "#{lib_path("foo-1.0")}" do
         gem 'foo'
       end
     G
 
-    bundle :install, :artifice => "endpoint"
+    bundle :install, artifice: "endpoint"
 
     expect(the_bundle).to include_gems("rails 2.3.2")
   end
@@ -93,13 +93,13 @@ RSpec.describe "gemcutter's dependency API" do
 
     gemfile <<-G
       source "#{source_uri}"
-      gem 'foo', :git => "#{file_uri_for(lib_path("foo-1.0"))}"
+      gem 'foo', :git => "#{lib_path("foo-1.0")}"
     G
 
-    bundle :install, :artifice => "endpoint"
+    bundle :install, artifice: "endpoint"
 
     bundle "config set --local deployment true"
-    bundle :install, :artifice => "endpoint"
+    bundle :install, artifice: "endpoint"
 
     expect(the_bundle).to include_gems("rails 2.3.2")
   end
@@ -108,35 +108,35 @@ RSpec.describe "gemcutter's dependency API" do
     build_git "foo"
     gemfile <<-G
       source "#{source_uri}"
-      gem 'foo', :git => "#{file_uri_for(lib_path("foo-1.0"))}"
+      gem 'foo', :git => "#{lib_path("foo-1.0")}"
     G
 
-    bundle "install", :artifice => "endpoint"
+    bundle "install", artifice: "endpoint"
     bundle "config set --local deployment true"
-    bundle :install, :artifice => "endpoint"
+    bundle :install, artifice: "endpoint"
 
     expect(the_bundle).to include_gems("foo 1.0")
   end
 
   it "falls back when the API errors out" do
-    simulate_platform x86_mswin32
-
-    build_repo2 do
-      # The rcov gem is platform mswin32, but has no arch
-      build_gem "rcov" do |s|
-        s.platform = Gem::Platform.new([nil, "mswin32", nil])
-        s.write "lib/rcov.rb", "RCOV = '1.0.0'"
+    simulate_platform x86_mswin32 do
+      build_repo2 do
+        # The rcov gem is platform mswin32, but has no arch
+        build_gem "rcov" do |s|
+          s.platform = Gem::Platform.new([nil, "mswin32", nil])
+          s.write "lib/rcov.rb", "RCOV = '1.0.0'"
+        end
       end
+
+      gemfile <<-G
+        source "#{source_uri}"
+        gem "rcov"
+      G
+
+      bundle :install, artifice: "windows", env: { "BUNDLER_SPEC_GEM_REPO" => gem_repo2.to_s }
+      expect(out).to include("Fetching source index from #{source_uri}")
+      expect(the_bundle).to include_gems "rcov 1.0.0"
     end
-
-    gemfile <<-G
-      source "#{source_uri}"
-      gem "rcov"
-    G
-
-    bundle :install, :artifice => "windows", :env => { "BUNDLER_SPEC_GEM_REPO" => gem_repo2.to_s }
-    expect(out).to include("Fetching source index from #{source_uri}")
-    expect(the_bundle).to include_gems "rcov 1.0.0"
   end
 
   it "falls back when hitting the Gemcutter Dependency Limit" do
@@ -147,10 +147,10 @@ RSpec.describe "gemcutter's dependency API" do
       gem "actionmailer"
       gem "activeresource"
       gem "thin"
-      gem "rack"
+      gem "myrack"
       gem "rails"
     G
-    bundle :install, :artifice => "endpoint_fallback"
+    bundle :install, artifice: "endpoint_fallback"
     expect(out).to include("Fetching source index from #{source_uri}")
 
     expect(the_bundle).to include_gems(
@@ -160,7 +160,7 @@ RSpec.describe "gemcutter's dependency API" do
       "activeresource 2.3.2",
       "activesupport 2.3.2",
       "thin 1.0.0",
-      "rack 1.0.0",
+      "myrack 1.0.0",
       "rails 2.3.2"
     )
   end
@@ -168,39 +168,39 @@ RSpec.describe "gemcutter's dependency API" do
   it "falls back when Gemcutter API doesn't return proper Marshal format" do
     gemfile <<-G
       source "#{source_uri}"
-      gem "rack"
+      gem "myrack"
     G
 
-    bundle :install, :verbose => true, :artifice => "endpoint_marshal_fail"
+    bundle :install, verbose: true, artifice: "endpoint_marshal_fail"
     expect(out).to include("could not fetch from the dependency API, trying the full index")
-    expect(the_bundle).to include_gems "rack 1.0.0"
+    expect(the_bundle).to include_gems "myrack 1.0.0"
   end
 
   it "falls back when the API URL returns 403 Forbidden" do
     gemfile <<-G
       source "#{source_uri}"
-      gem "rack"
+      gem "myrack"
     G
 
-    bundle :install, :verbose => true, :artifice => "endpoint_api_forbidden"
+    bundle :install, verbose: true, artifice: "endpoint_api_forbidden"
     expect(out).to include("Fetching source index from #{source_uri}")
-    expect(the_bundle).to include_gems "rack 1.0.0"
+    expect(the_bundle).to include_gems "myrack 1.0.0"
   end
 
   it "handles host redirects" do
     gemfile <<-G
       source "#{source_uri}"
-      gem "rack"
+      gem "myrack"
     G
 
-    bundle :install, :artifice => "endpoint_host_redirect"
-    expect(the_bundle).to include_gems "rack 1.0.0"
+    bundle :install, artifice: "endpoint_host_redirect"
+    expect(the_bundle).to include_gems "myrack 1.0.0"
   end
 
-  it "handles host redirects without Net::HTTP::Persistent" do
+  it "handles host redirects without Gem::Net::HTTP::Persistent" do
     gemfile <<-G
       source "#{source_uri}"
-      gem "rack"
+      gem "myrack"
     G
 
     FileUtils.mkdir_p lib_path
@@ -216,18 +216,18 @@ RSpec.describe "gemcutter's dependency API" do
       H
     end
 
-    bundle :install, :artifice => "endpoint_host_redirect", :requires => [lib_path("disable_net_http_persistent.rb")]
+    bundle :install, artifice: "endpoint_host_redirect", requires: [lib_path("disable_net_http_persistent.rb")]
     expect(out).to_not match(/Too many redirects/)
-    expect(the_bundle).to include_gems "rack 1.0.0"
+    expect(the_bundle).to include_gems "myrack 1.0.0"
   end
 
   it "timeouts when Bundler::Fetcher redirects too much" do
     gemfile <<-G
       source "#{source_uri}"
-      gem "rack"
+      gem "myrack"
     G
 
-    bundle :install, :artifice => "endpoint_redirect", :raise_on_error => false
+    bundle :install, artifice: "endpoint_redirect", raise_on_error: false
     expect(err).to match(/Too many redirects/)
   end
 
@@ -235,27 +235,27 @@ RSpec.describe "gemcutter's dependency API" do
     it "should use the modern index for install" do
       gemfile <<-G
         source "#{source_uri}"
-        gem "rack"
+        gem "myrack"
       G
 
-      bundle "install --full-index", :artifice => "endpoint"
+      bundle "install --full-index", artifice: "endpoint"
       expect(out).to include("Fetching source index from #{source_uri}")
-      expect(the_bundle).to include_gems "rack 1.0.0"
+      expect(the_bundle).to include_gems "myrack 1.0.0"
     end
 
     it "should use the modern index for update" do
       gemfile <<-G
         source "#{source_uri}"
-        gem "rack"
+        gem "myrack"
       G
 
-      bundle "update --full-index", :artifice => "endpoint", :all => true
+      bundle "update --full-index", artifice: "endpoint", all: true
       expect(out).to include("Fetching source index from #{source_uri}")
-      expect(the_bundle).to include_gems "rack 1.0.0"
+      expect(the_bundle).to include_gems "myrack 1.0.0"
     end
   end
 
-  it "fetches again when more dependencies are found in subsequent sources", :bundler => "< 3" do
+  it "fetches again when more dependencies are found in subsequent sources", bundler: "< 3" do
     build_repo2 do
       build_gem "back_deps" do |s|
         s.add_dependency "foo"
@@ -269,7 +269,7 @@ RSpec.describe "gemcutter's dependency API" do
       gem "back_deps"
     G
 
-    bundle :install, :artifice => "endpoint_extra"
+    bundle :install, artifice: "endpoint_extra"
     expect(the_bundle).to include_gems "back_deps 1.0", "foo 1.0"
   end
 
@@ -288,33 +288,33 @@ RSpec.describe "gemcutter's dependency API" do
       end
     G
 
-    bundle :install, :artifice => "endpoint_extra"
+    bundle :install, artifice: "endpoint_extra"
     expect(the_bundle).to include_gems "back_deps 1.0", "foo 1.0"
   end
 
   it "fetches gem versions even when those gems are already installed" do
     gemfile <<-G
       source "#{source_uri}"
-      gem "rack", "1.0.0"
+      gem "myrack", "1.0.0"
     G
-    bundle :install, :artifice => "endpoint_extra_api"
+    bundle :install, artifice: "endpoint_extra_api"
 
     build_repo4 do
-      build_gem "rack", "1.2" do |s|
-        s.executables = "rackup"
+      build_gem "myrack", "1.2" do |s|
+        s.executables = "myrackup"
       end
     end
 
     gemfile <<-G
       source "#{source_uri}" do; end
       source "#{source_uri}/extra"
-      gem "rack", "1.2"
+      gem "myrack", "1.2"
     G
-    bundle :install, :artifice => "endpoint_extra_api"
-    expect(the_bundle).to include_gems "rack 1.2"
+    bundle :install, artifice: "endpoint_extra_api"
+    expect(the_bundle).to include_gems "myrack 1.2"
   end
 
-  it "considers all possible versions of dependencies from all api gem sources", :bundler => "< 3" do
+  it "considers all possible versions of dependencies from all api gem sources", bundler: "< 3" do
     # In this scenario, the gem "somegem" only exists in repo4.  It depends on specific version of activesupport that
     # exists only in repo1.  There happens also be a version of activesupport in repo4, but not the one that version 1.0.0
     # of somegem wants. This test makes sure that bundler actually finds version 1.2.3 of active support in the other
@@ -332,7 +332,7 @@ RSpec.describe "gemcutter's dependency API" do
       gem 'somegem', '1.0.0'
     G
 
-    bundle :install, :artifice => "endpoint_extra_api"
+    bundle :install, artifice: "endpoint_extra_api"
 
     expect(the_bundle).to include_gems "somegem 1.0.0"
     expect(the_bundle).to include_gems "activesupport 1.2.3"
@@ -353,13 +353,13 @@ RSpec.describe "gemcutter's dependency API" do
       end
     G
 
-    bundle :install, :artifice => "endpoint_extra"
+    bundle :install, artifice: "endpoint_extra"
 
     expect(out).to include("Fetching gem metadata from http://localgemserver.test/.")
     expect(out).to include("Fetching source index from http://localgemserver.test/extra")
   end
 
-  it "does not fetch every spec when doing back deps", :bundler => "< 3" do
+  it "does not fetch every spec when doing back deps", bundler: "< 3" do
     build_repo2 do
       build_gem "back_deps" do |s|
         s.add_dependency "foo"
@@ -369,7 +369,7 @@ RSpec.describe "gemcutter's dependency API" do
       FileUtils.rm_rf Dir[gem_repo2("gems/foo-*.gem")]
     end
 
-    install_gemfile <<-G, :artifice => "endpoint_extra_missing", :env => env_for_missing_prerelease_default_gem_activation
+    install_gemfile <<-G, artifice: "endpoint_extra_missing"
       source "#{source_uri}"
       source "#{source_uri}/extra"
       gem "back_deps"
@@ -388,7 +388,7 @@ RSpec.describe "gemcutter's dependency API" do
       FileUtils.rm_rf Dir[gem_repo2("gems/foo-*.gem")]
     end
 
-    install_gemfile <<-G, :artifice => "endpoint_extra_missing", :env => env_for_missing_prerelease_default_gem_activation
+    install_gemfile <<-G, artifice: "endpoint_extra_missing"
       source "#{source_uri}"
       source "#{source_uri}/extra" do
         gem "back_deps"
@@ -398,7 +398,7 @@ RSpec.describe "gemcutter's dependency API" do
     expect(the_bundle).to include_gems "back_deps 1.0"
   end
 
-  it "fetches again when more dependencies are found in subsequent sources using deployment mode", :bundler => "< 3" do
+  it "fetches again when more dependencies are found in subsequent sources using deployment mode", bundler: "< 3" do
     build_repo2 do
       build_gem "back_deps" do |s|
         s.add_dependency "foo"
@@ -412,9 +412,9 @@ RSpec.describe "gemcutter's dependency API" do
       gem "back_deps"
     G
 
-    bundle :install, :artifice => "endpoint_extra"
+    bundle :install, artifice: "endpoint_extra"
     bundle "config set --local deployment true"
-    bundle :install, :artifice => "endpoint_extra"
+    bundle :install, artifice: "endpoint_extra"
     expect(the_bundle).to include_gems "back_deps 1.0"
   end
 
@@ -433,9 +433,9 @@ RSpec.describe "gemcutter's dependency API" do
       end
     G
 
-    bundle :install, :artifice => "endpoint_extra"
+    bundle :install, artifice: "endpoint_extra"
     bundle "config set --local deployment true"
-    bundle "install", :artifice => "endpoint_extra"
+    bundle "install", artifice: "endpoint_extra"
     expect(the_bundle).to include_gems "back_deps 1.0"
   end
 
@@ -445,7 +445,7 @@ RSpec.describe "gemcutter's dependency API" do
       build_gem "foo", "2.0"
     end
 
-    install_gemfile <<-G, :artifice => "endpoint", :env => { "BUNDLER_SPEC_GEM_REPO" => gem_repo2.to_s }, :verbose => true
+    install_gemfile <<-G, artifice: "endpoint", env: { "BUNDLER_SPEC_GEM_REPO" => gem_repo2.to_s }, verbose: true
       source "#{source_uri}"
 
       gem "foo"
@@ -468,70 +468,70 @@ RSpec.describe "gemcutter's dependency API" do
       gem "bundler_dep"
     G
 
-    bundle :install, :artifice => "endpoint", :env => { "BUNDLER_SPEC_GEM_REPO" => gem_repo2.to_s }
+    bundle :install, artifice: "endpoint", env: { "BUNDLER_SPEC_GEM_REPO" => gem_repo2.to_s }
     expect(out).to include("Fetching gem metadata from #{source_uri}")
   end
 
-  it "installs the binstubs", :bundler => "< 3" do
+  it "installs the binstubs", bundler: "< 3" do
     gemfile <<-G
       source "#{source_uri}"
-      gem "rack"
+      gem "myrack"
     G
 
-    bundle "install --binstubs", :artifice => "endpoint"
+    bundle "install --binstubs", artifice: "endpoint"
 
-    gembin "rackup"
+    gembin "myrackup"
     expect(out).to eq("1.0.0")
   end
 
-  it "installs the bins when using --path and uses autoclean", :bundler => "< 3" do
+  it "installs the bins when using --path and uses autoclean", bundler: "< 3" do
     gemfile <<-G
       source "#{source_uri}"
-      gem "rack"
+      gem "myrack"
     G
 
-    bundle "install --path vendor/bundle", :artifice => "endpoint"
+    bundle "install --path vendor/bundle", artifice: "endpoint"
 
-    expect(vendored_gems("bin/rackup")).to exist
+    expect(vendored_gems("bin/myrackup")).to exist
   end
 
-  it "installs the bins when using --path and uses bundle clean", :bundler => "< 3" do
+  it "installs the bins when using --path and uses bundle clean", bundler: "< 3" do
     gemfile <<-G
       source "#{source_uri}"
-      gem "rack"
+      gem "myrack"
     G
 
-    bundle "install --path vendor/bundle --no-clean", :artifice => "endpoint"
+    bundle "install --path vendor/bundle --no-clean", artifice: "endpoint"
 
-    expect(vendored_gems("bin/rackup")).to exist
+    expect(vendored_gems("bin/myrackup")).to exist
   end
 
   it "prints post_install_messages" do
     gemfile <<-G
       source "#{source_uri}"
-      gem 'rack-obama'
+      gem 'myrack-obama'
     G
 
-    bundle :install, :artifice => "endpoint"
-    expect(out).to include("Post-install message from rack:")
+    bundle :install, artifice: "endpoint"
+    expect(out).to include("Post-install message from myrack:")
   end
 
   it "should display the post install message for a dependency" do
     gemfile <<-G
       source "#{source_uri}"
-      gem 'rack_middleware'
+      gem 'myrack_middleware'
     G
 
-    bundle :install, :artifice => "endpoint"
-    expect(out).to include("Post-install message from rack:")
-    expect(out).to include("Rack's post install message")
+    bundle :install, artifice: "endpoint"
+    expect(out).to include("Post-install message from myrack:")
+    expect(out).to include("Myrack's post install message")
   end
 
   context "when using basic authentication" do
     let(:user)     { "user" }
     let(:password) { "pass" }
     let(:basic_auth_source_uri) do
-      uri          = Bundler::URI.parse(source_uri)
+      uri          = Gem::URI.parse(source_uri)
       uri.user     = user
       uri.password = password
 
@@ -541,82 +541,82 @@ RSpec.describe "gemcutter's dependency API" do
     it "passes basic authentication details and strips out creds" do
       gemfile <<-G
         source "#{basic_auth_source_uri}"
-        gem "rack"
+        gem "myrack"
       G
 
-      bundle :install, :artifice => "endpoint_basic_authentication"
+      bundle :install, artifice: "endpoint_basic_authentication"
       expect(out).not_to include("#{user}:#{password}")
-      expect(the_bundle).to include_gems "rack 1.0.0"
+      expect(the_bundle).to include_gems "myrack 1.0.0"
     end
 
     it "passes basic authentication details and strips out creds also in verbose mode" do
       gemfile <<-G
         source "#{basic_auth_source_uri}"
-        gem "rack"
+        gem "myrack"
       G
 
-      bundle :install, :verbose => true, :artifice => "endpoint_basic_authentication"
+      bundle :install, verbose: true, artifice: "endpoint_basic_authentication"
       expect(out).not_to include("#{user}:#{password}")
-      expect(the_bundle).to include_gems "rack 1.0.0"
+      expect(the_bundle).to include_gems "myrack 1.0.0"
     end
 
     it "strips http basic authentication creds for modern index" do
       gemfile <<-G
         source "#{basic_auth_source_uri}"
-        gem "rack"
+        gem "myrack"
       G
 
-      bundle :install, :artifice => "endpoint_marshal_fail_basic_authentication"
+      bundle :install, artifice: "endpoint_marshal_fail_basic_authentication"
       expect(out).not_to include("#{user}:#{password}")
-      expect(the_bundle).to include_gems "rack 1.0.0"
+      expect(the_bundle).to include_gems "myrack 1.0.0"
     end
 
     it "strips http basic auth creds when it can't reach the server" do
       gemfile <<-G
         source "#{basic_auth_source_uri}"
-        gem "rack"
+        gem "myrack"
       G
 
-      bundle :install, :artifice => "endpoint_500", :raise_on_error => false
+      bundle :install, artifice: "endpoint_500", raise_on_error: false
       expect(out).not_to include("#{user}:#{password}")
     end
 
-    it "strips http basic auth creds when warning about ambiguous sources", :bundler => "< 3" do
+    it "strips http basic auth creds when warning about ambiguous sources", bundler: "< 3" do
       gemfile <<-G
         source "#{basic_auth_source_uri}"
         source "#{file_uri_for(gem_repo1)}"
-        gem "rack"
+        gem "myrack"
       G
 
-      bundle :install, :artifice => "endpoint_basic_authentication"
-      expect(err).to include("Warning: the gem 'rack' was found in multiple sources.")
+      bundle :install, artifice: "endpoint_basic_authentication"
+      expect(err).to include("Warning: the gem 'myrack' was found in multiple sources.")
       expect(err).not_to include("#{user}:#{password}")
-      expect(the_bundle).to include_gems "rack 1.0.0"
+      expect(the_bundle).to include_gems "myrack 1.0.0"
     end
 
     it "does not pass the user / password to different hosts on redirect" do
       gemfile <<-G
         source "#{basic_auth_source_uri}"
-        gem "rack"
+        gem "myrack"
       G
 
-      bundle :install, :artifice => "endpoint_creds_diff_host"
-      expect(the_bundle).to include_gems "rack 1.0.0"
+      bundle :install, artifice: "endpoint_creds_diff_host"
+      expect(the_bundle).to include_gems "myrack 1.0.0"
     end
 
     describe "with host including dashes" do
       before do
         gemfile <<-G
           source "http://local-gemserver.test"
-          gem "rack"
+          gem "myrack"
         G
       end
 
       it "reads authentication details from a valid ENV variable" do
-        bundle :install, :artifice => "endpoint_strict_basic_authentication", :env => { "BUNDLE_LOCAL___GEMSERVER__TEST" => "#{user}:#{password}" }
+        bundle :install, artifice: "endpoint_strict_basic_authentication", env: { "BUNDLE_LOCAL___GEMSERVER__TEST" => "#{user}:#{password}" }
 
         expect(out).to include("Fetching gem metadata from http://local-gemserver.test")
-        expect(the_bundle).to include_gems "rack 1.0.0"
+        expect(the_bundle).to include_gems "myrack 1.0.0"
       end
     end
 
@@ -624,57 +624,57 @@ RSpec.describe "gemcutter's dependency API" do
       before do
         gemfile <<-G
           source "#{source_uri}"
-          gem "rack"
+          gem "myrack"
         G
       end
 
       it "reads authentication details by host name from bundle config" do
         bundle "config set #{source_hostname} #{user}:#{password}"
 
-        bundle :install, :artifice => "endpoint_strict_basic_authentication"
+        bundle :install, artifice: "endpoint_strict_basic_authentication"
 
         expect(out).to include("Fetching gem metadata from #{source_uri}")
-        expect(the_bundle).to include_gems "rack 1.0.0"
+        expect(the_bundle).to include_gems "myrack 1.0.0"
       end
 
       it "reads authentication details by full url from bundle config" do
         # The trailing slash is necessary here; Fetcher canonicalizes the URI.
         bundle "config set #{source_uri}/ #{user}:#{password}"
 
-        bundle :install, :artifice => "endpoint_strict_basic_authentication"
+        bundle :install, artifice: "endpoint_strict_basic_authentication"
 
         expect(out).to include("Fetching gem metadata from #{source_uri}")
-        expect(the_bundle).to include_gems "rack 1.0.0"
+        expect(the_bundle).to include_gems "myrack 1.0.0"
       end
 
       it "should use the API" do
         bundle "config set #{source_hostname} #{user}:#{password}"
-        bundle :install, :artifice => "endpoint_strict_basic_authentication"
+        bundle :install, artifice: "endpoint_strict_basic_authentication"
         expect(out).to include("Fetching gem metadata from #{source_uri}")
-        expect(the_bundle).to include_gems "rack 1.0.0"
+        expect(the_bundle).to include_gems "myrack 1.0.0"
       end
 
       it "prefers auth supplied in the source uri" do
         gemfile <<-G
           source "#{basic_auth_source_uri}"
-          gem "rack"
+          gem "myrack"
         G
 
         bundle "config set #{source_hostname} otheruser:wrong"
 
-        bundle :install, :artifice => "endpoint_strict_basic_authentication"
-        expect(the_bundle).to include_gems "rack 1.0.0"
+        bundle :install, artifice: "endpoint_strict_basic_authentication"
+        expect(the_bundle).to include_gems "myrack 1.0.0"
       end
 
       it "shows instructions if auth is not provided for the source" do
-        bundle :install, :artifice => "endpoint_strict_basic_authentication", :raise_on_error => false
+        bundle :install, artifice: "endpoint_strict_basic_authentication", raise_on_error: false
         expect(err).to include("bundle config set --global #{source_hostname} username:password")
       end
 
       it "fails if authentication has already been provided, but failed" do
         bundle "config set #{source_hostname} #{user}:wrong"
 
-        bundle :install, :artifice => "endpoint_strict_basic_authentication", :raise_on_error => false
+        bundle :install, artifice: "endpoint_strict_basic_authentication", raise_on_error: false
         expect(err).to include("Bad username or password")
       end
     end
@@ -685,11 +685,11 @@ RSpec.describe "gemcutter's dependency API" do
       it "passes basic authentication details" do
         gemfile <<-G
           source "#{basic_auth_source_uri}"
-          gem "rack"
+          gem "myrack"
         G
 
-        bundle :install, :artifice => "endpoint_basic_authentication"
-        expect(the_bundle).to include_gems "rack 1.0.0"
+        bundle :install, artifice: "endpoint_basic_authentication"
+        expect(the_bundle).to include_gems "myrack 1.0.0"
       end
     end
   end
@@ -710,10 +710,10 @@ RSpec.describe "gemcutter's dependency API" do
     it "explains what to do to get it" do
       gemfile <<-G
         source "#{source_uri.gsub(/http/, "https")}"
-        gem "rack"
+        gem "myrack"
       G
 
-      bundle :install, :env => { "RUBYOPT" => opt_add("-I#{bundled_app("broken_ssl")}", ENV["RUBYOPT"]) }, :raise_on_error => false
+      bundle :install, artifice: "fail", env: { "RUBYOPT" => opt_add("-I#{bundled_app("broken_ssl")}", ENV["RUBYOPT"]) }, raise_on_error: false
       expect(err).to include("OpenSSL")
     end
   end
@@ -723,17 +723,17 @@ RSpec.describe "gemcutter's dependency API" do
       # Install a monkeypatch that reproduces the effects of openssl raising
       # a certificate validation error when RubyGems tries to connect.
       gemfile <<-G
-        class Net::HTTP
+        class Gem::Net::HTTP
           def start
             raise OpenSSL::SSL::SSLError, "certificate verify failed"
           end
         end
 
         source "#{source_uri.gsub(/http/, "https")}"
-        gem "rack"
+        gem "myrack"
       G
 
-      bundle :install, :raise_on_error => false
+      bundle :install, raise_on_error: false
       expect(err).to match(/could not verify the SSL certificate/i)
     end
   end
@@ -741,16 +741,16 @@ RSpec.describe "gemcutter's dependency API" do
   context ".gemrc with sources is present" do
     it "uses other sources declared in the Gemfile" do
       File.open(home(".gemrc"), "w") do |file|
-        file.puts({ :sources => ["https://rubygems.org"] }.to_yaml)
+        file.puts({ sources: ["https://rubygems.org"] }.to_yaml)
       end
 
       begin
         gemfile <<-G
           source "#{source_uri}"
-          gem 'rack'
+          gem 'myrack'
         G
 
-        bundle "install", :artifice => "endpoint_marshal_fail"
+        bundle "install", artifice: "endpoint_marshal_fail"
       ensure
         home(".gemrc").rmtree
       end
