@@ -40,7 +40,7 @@ class CompactIndexAPI < Endpoint
     end
 
     def requested_range_for(response_body)
-      ranges = Rack::Utils.byte_ranges(env, response_body.bytesize)
+      ranges = Rack::Utils.get_byte_ranges(env["HTTP_RANGE"], response_body.bytesize)
 
       if ranges
         status 206
@@ -68,7 +68,10 @@ class CompactIndexAPI < Endpoint
       @gems[gem_repo] ||= begin
         specs = Bundler::Deprecate.skip_during do
           %w[specs.4.8 prerelease_specs.4.8].map do |filename|
-            Marshal.load(File.open(gem_repo.join(filename)).read).map do |name, version, platform|
+            spec_index = gem_repo.join(filename)
+            next [] unless File.exist?(spec_index)
+
+            Marshal.load(File.binread(spec_index)).map do |name, version, platform|
               load_spec(name, version, platform, gem_repo)
             end
           end.flatten
