@@ -192,22 +192,11 @@ describe "String#gsub with pattern and replacement" do
     -> { "hello".gsub(/[aeiou]/, nil)           }.should raise_error(TypeError)
   end
 
-  ruby_version_is ''...'3.0' do
-    it "returns subclass instances when called on a subclass" do
-      StringSpecs::MyString.new("").gsub(//, "").should be_an_instance_of(StringSpecs::MyString)
-      StringSpecs::MyString.new("").gsub(/foo/, "").should be_an_instance_of(StringSpecs::MyString)
-      StringSpecs::MyString.new("foo").gsub(/foo/, "").should be_an_instance_of(StringSpecs::MyString)
-      StringSpecs::MyString.new("foo").gsub("foo", "").should be_an_instance_of(StringSpecs::MyString)
-    end
-  end
-
-  ruby_version_is '3.0' do
-    it "returns String instances when called on a subclass" do
-      StringSpecs::MyString.new("").gsub(//, "").should be_an_instance_of(String)
-      StringSpecs::MyString.new("").gsub(/foo/, "").should be_an_instance_of(String)
-      StringSpecs::MyString.new("foo").gsub(/foo/, "").should be_an_instance_of(String)
-      StringSpecs::MyString.new("foo").gsub("foo", "").should be_an_instance_of(String)
-    end
+  it "returns String instances when called on a subclass" do
+    StringSpecs::MyString.new("").gsub(//, "").should be_an_instance_of(String)
+    StringSpecs::MyString.new("").gsub(/foo/, "").should be_an_instance_of(String)
+    StringSpecs::MyString.new("foo").gsub(/foo/, "").should be_an_instance_of(String)
+    StringSpecs::MyString.new("foo").gsub("foo", "").should be_an_instance_of(String)
   end
 
   it "sets $~ to MatchData of last match and nil when there's none" do
