@@ -18,7 +18,6 @@
 #include "internal/proc.h"
 #include "internal/struct.h"
 #include "internal/symbol.h"
-#include "transient_heap.h"
 #include "vm_core.h"
 #include "builtin.h"
 
@@ -53,7 +52,8 @@ struct_ivar_get(VALUE c, ID id)
         RUBY_ASSERT(RB_TYPE_P(c, T_CLASS));
         ivar = rb_attr_get(c, id);
         if (!NIL_P(ivar)) {
-            return rb_ivar_set(orig, id, ivar);
+            if (!OBJ_FROZEN(orig)) rb_ivar_set(orig, id, ivar);
+            return ivar;
         }
     }
 }
@@ -413,18 +413,15 @@ struct_make_members_list(va_list ar)
 {
     char *mem;
     VALUE ary, list = rb_ident_hash_new();
-    st_table *tbl = RHASH_TBL_RAW(list);
-
     RBASIC_CLEAR_CLASS(list);
-    OBJ_WB_UNPROTECT(list);
     while ((mem = va_arg(ar, char*)) != 0) {
         VALUE sym = rb_sym_intern_ascii_cstr(mem);
-        if (st_insert(tbl, sym, Qtrue)) {
+        if (RTEST(rb_hash_has_key(list, sym))) {
             rb_raise(rb_eArgError, "duplicate member: %s", mem);
         }
+        rb_hash_aset(list, sym, Qtrue);
     }
     ary = rb_hash_keys(list);
-    st_clear(tbl);
     RBASIC_CLEAR_CLASS(ary);
     OBJ_FREEZE_RAW(ary);
     return ary;
@@ -520,8 +517,8 @@ rb_struct_define_under(VALUE outer, const char *name, ...)
 
 /*
  *  call-seq:
- *    Struct.new(*member_names, keyword_init: false){|Struct_subclass| ... } -> Struct_subclass
- *    Struct.new(class_name, *member_names, keyword_init: false){|Struct_subclass| ... } -> Struct_subclass
+ *    Struct.new(*member_names, keyword_init: nil){|Struct_subclass| ... } -> Struct_subclass
+ *    Struct.new(class_name, *member_names, keyword_init: nil){|Struct_subclass| ... } -> Struct_subclass
  *    Struct_subclass.new(*member_names) -> Struct_subclass_instance
  *    Struct_subclass.new(**member_names) -> Struct_subclass_instance
  *
@@ -570,7 +567,7 @@ rb_struct_define_under(VALUE outer, const char *name, ...)
  *
  *  <b>Member Names</b>
  *
- *  \Symbol arguments +member_names+
+ *  Symbol arguments +member_names+
  *  determines the members of the new subclass:
  *
  *    Struct.new(:foo, :bar).members        # => [:foo, :bar]
@@ -604,8 +601,6 @@ rb_struct_define_under(VALUE outer, const char *name, ...)
  *      Foo.new(foo: 0, bar: 1, baz: 2)
  *      # Raises ArgumentError: unknown keywords: baz
  *
- *    \Method <tt>::[]</tt> is an alias for method <tt>::new</tt>.
- *
  *  - \Method <tt>:inspect</tt> returns a string representation of the subclass:
  *
  *      Foo.inspect
@@ -647,18 +642,14 @@ rb_struct_define_under(VALUE outer, const char *name, ...)
 static VALUE
 rb_struct_s_def(int argc, VALUE *argv, VALUE klass)
 {
-    VALUE name, rest, keyword_init = Qnil;
+    VALUE name = Qnil, rest, keyword_init = Qnil;
     long i;
     VALUE st;
-    st_table *tbl;
     VALUE opt;
 
-    argc = rb_scan_args(argc, argv, "1*:", NULL, NULL, &opt);
-    name = argv[0];
-    if (SYMBOL_P(name)) {
-        name = Qnil;
-    }
-    else {
+    argc = rb_scan_args(argc, argv, "0*:", NULL, &opt);
+    if (argc >= 1 && !SYMBOL_P(argv[0])) {
+        name = argv[0];
         --argc;
         ++argv;
     }
@@ -680,19 +671,17 @@ rb_struct_s_def(int argc, VALUE *argv, VALUE klass)
 
     rest = rb_ident_hash_new();
     RBASIC_CLEAR_CLASS(rest);
-    OBJ_WB_UNPROTECT(rest);
-    tbl = RHASH_TBL_RAW(rest);
     for (i=0; i<argc; i++) {
         VALUE mem = rb_to_symbol(argv[i]);
         if (rb_is_attrset_sym(mem)) {
             rb_raise(rb_eArgError, "invalid struct member: %"PRIsVALUE, mem);
         }
-        if (st_insert(tbl, mem, Qtrue)) {
+        if (RTEST(rb_hash_has_key(rest, mem))) {
             rb_raise(rb_eArgError, "duplicate member: %"PRIsVALUE, mem);
         }
+        rb_hash_aset(rest, mem, Qtrue);
     }
     rest = rb_hash_keys(rest);
-    st_clear(tbl);
     RBASIC_CLEAR_CLASS(rest);
     OBJ_FREEZE_RAW(rest);
     if (NIL_P(name)) {
@@ -814,60 +803,34 @@ rb_struct_initialize(VALUE self, VALUE values)
 static VALUE *
 struct_heap_alloc(VALUE st, size_t len)
 {
-    VALUE *ptr = rb_transient_heap_alloc((VALUE)st, sizeof(VALUE) * len);
-
-    if (ptr) {
-        RSTRUCT_TRANSIENT_SET(st);
-        return ptr;
-    }
-    else {
-        RSTRUCT_TRANSIENT_UNSET(st);
-        return ALLOC_N(VALUE, len);
-    }
+    return ALLOC_N(VALUE, len);
 }
 
-#if USE_TRANSIENT_HEAP
-void
-rb_struct_transient_heap_evacuate(VALUE obj, int promote)
-{
-    if (RSTRUCT_TRANSIENT_P(obj)) {
-        const VALUE *old_ptr = rb_struct_const_heap_ptr(obj);
-        VALUE *new_ptr;
-        long len = RSTRUCT_LEN(obj);
-
-        if (promote) {
-            new_ptr = ALLOC_N(VALUE, len);
-            FL_UNSET_RAW(obj, RSTRUCT_TRANSIENT_FLAG);
-        }
-        else {
-            new_ptr = struct_heap_alloc(obj, len);
-        }
-        MEMCPY(new_ptr, old_ptr, VALUE, len);
-        RSTRUCT(obj)->as.heap.ptr = new_ptr;
-    }
-}
-#endif
-
 static VALUE
 struct_alloc(VALUE klass)
 {
-    long n;
-    NEWOBJ_OF(st, struct RStruct, klass, T_STRUCT | (RGENGC_WB_PROTECTED_STRUCT ? FL_WB_PROTECTED : 0));
+    long n = num_members(klass);
+    size_t embedded_size = offsetof(struct RStruct, as.ary) + (sizeof(VALUE) * n);
+    VALUE flags = T_STRUCT | (RGENGC_WB_PROTECTED_STRUCT ? FL_WB_PROTECTED : 0);
 
-    n = num_members(klass);
+    if (n > 0 && rb_gc_size_allocatable_p(embedded_size)) {
+        flags |= n << RSTRUCT_EMBED_LEN_SHIFT;
+
+        NEWOBJ_OF(st, struct RStruct, klass, flags, embedded_size, 0);
 
-    if (0 < n && n <= RSTRUCT_EMBED_LEN_MAX) {
-        RBASIC(st)->flags &= ~RSTRUCT_EMBED_LEN_MASK;
-        RBASIC(st)->flags |= n << RSTRUCT_EMBED_LEN_SHIFT;
         rb_mem_clear((VALUE *)st->as.ary, n);
+
+        return (VALUE)st;
     }
     else {
+        NEWOBJ_OF(st, struct RStruct, klass, flags, sizeof(struct RStruct), 0);
+
         st->as.heap.ptr = struct_heap_alloc((VALUE)st, n);
         rb_mem_clear((VALUE *)st->as.heap.ptr, n);
         st->as.heap.len = n;
-    }
 
-    return (VALUE)st;
+        return (VALUE)st;
+    }
 }
 
 VALUE
@@ -1040,8 +1003,6 @@ inspect_struct(VALUE s, VALUE prefix, int recur)
  *    joe = Customer.new("Joe Smith", "123 Maple, Anytown NC", 12345)
  *    joe.inspect # => "#<struct Customer name=\"Joe Smith\", address=\"123 Maple, Anytown NC\", zip=12345>"
  *
- *  Struct#to_s is an alias for Struct#inspect.
- *
  */
 
 static VALUE
@@ -1060,8 +1021,6 @@ rb_struct_inspect(VALUE s)
  *    joe = Customer.new("Joe Smith", "123 Maple, Anytown NC", 12345)
  *    joe.to_a # => ["Joe Smith", "123 Maple, Anytown NC", 12345]
  *
- *  Struct#values and Struct#deconstruct are aliases for Struct#to_a.
- *
  *  Related: #members.
  */
 
@@ -1383,8 +1342,6 @@ rb_struct_values_at(int argc, VALUE *argv, VALUE s)
  *    a # => [12345]
  *
  *  With no block given, returns an Enumerator.
- *
- *  Struct#filter is an alias for Struct#select.
  */
 
 static VALUE
@@ -1543,7 +1500,6 @@ rb_struct_eql(VALUE s, VALUE s2)
  *    joe = Customer.new("Joe Smith", "123 Maple, Anytown NC", 12345)
  *    joe.size #=> 3
  *
- *  Struct#length is an alias for Struct#size.
  */
 
 VALUE
@@ -1625,7 +1581,7 @@ rb_struct_dig(int argc, VALUE *argv, VALUE self)
  *     distance.unit #=> "km"
  *
  *  Constructed object also has a reasonable definitions of #==
- *  operator, #to_h hash conversion, and #deconstruct/#deconstruct_keys
+ *  operator, #to_h hash conversion, and #deconstruct / #deconstruct_keys
  *  to be used in pattern matching.
  *
  *  ::define method accepts an optional block and evaluates it in
@@ -1715,23 +1671,20 @@ rb_data_s_def(int argc, VALUE *argv, VALUE klass)
     VALUE rest;
     long i;
     VALUE data_class;
-    st_table *tbl;
 
     rest = rb_ident_hash_new();
     RBASIC_CLEAR_CLASS(rest);
-    OBJ_WB_UNPROTECT(rest);
-    tbl = RHASH_TBL_RAW(rest);
     for (i=0; i<argc; i++) {
         VALUE mem = rb_to_symbol(argv[i]);
         if (rb_is_attrset_sym(mem)) {
             rb_raise(rb_eArgError, "invalid data member: %"PRIsVALUE, mem);
         }
-        if (st_insert(tbl, mem, Qtrue)) {
+        if (RTEST(rb_hash_has_key(rest, mem))) {
             rb_raise(rb_eArgError, "duplicate member: %"PRIsVALUE, mem);
         }
+        rb_hash_aset(rest, mem, Qtrue);
     }
     rest = rb_hash_keys(rest);
-    st_clear(tbl);
     RBASIC_CLEAR_CLASS(rest);
     OBJ_FREEZE_RAW(rest);
     data_class = anonymous_struct(klass);
@@ -1743,6 +1696,20 @@ rb_data_s_def(int argc, VALUE *argv, VALUE klass)
     return data_class;
 }
 
+VALUE
+rb_data_define(VALUE super, ...)
+{
+    va_list ar;
+    VALUE ary;
+    va_start(ar, super);
+    ary = struct_make_members_list(ar);
+    va_end(ar);
+    if (!super) super = rb_cData;
+    VALUE klass = setup_data(anonymous_struct(super), ary);
+    rb_vm_add_root_module(klass);
+    return klass;
+}
+
 /*
  *  call-seq:
  *    DataClass::members -> array_of_symbols
@@ -1774,7 +1741,7 @@ rb_data_s_def(int argc, VALUE *argv, VALUE klass)
  *     Measure.new(amount: 1, unit: 'km')
  *     #=> #<data Measure amount=1, unit="km">
  *
- *     # Alternative shorter intialization with []
+ *     # Alternative shorter initialization with []
  *     Measure[1, 'km']
  *     #=> #<data Measure amount=1, unit="km">
  *     Measure[amount: 1, unit: 'km']
@@ -1834,10 +1801,12 @@ rb_data_initialize_m(int argc, const VALUE *argv, VALUE self)
     arg.self = self;
     arg.unknown_keywords = Qnil;
     rb_hash_foreach(argv[0], struct_hash_set_i, (VALUE)&arg);
+    // Freeze early before potentially raising, so that we don't leave an
+    // unfrozen copy on the heap, which could get exposed via ObjectSpace.
+    OBJ_FREEZE_RAW(self);
     if (arg.unknown_keywords != Qnil) {
         rb_exc_raise(rb_keyword_error_new("unknown", arg.unknown_keywords));
     }
-    OBJ_FREEZE_RAW(self);
     return Qnil;
 }
 
@@ -1889,22 +1858,9 @@ rb_data_with(int argc, const VALUE *argv, VALUE self)
         return self;
     }
 
-    VALUE copy = rb_obj_alloc(rb_obj_class(self));
-    rb_struct_init_copy(copy, self);
-
-    struct struct_hash_set_arg arg;
-    arg.self = copy;
-    arg.unknown_keywords = Qnil;
-    rb_hash_foreach(kwargs, struct_hash_set_i, (VALUE)&arg);
-    // Freeze early before potentially raising, so that we don't leave an
-    // unfrozen copy on the heap, which could get exposed via ObjectSpace.
-    RB_OBJ_FREEZE_RAW(copy);
-
-    if (arg.unknown_keywords != Qnil) {
-        rb_exc_raise(rb_keyword_error_new("unknown", arg.unknown_keywords));
-    }
-
-    return copy;
+    VALUE h = rb_struct_to_h(self);
+    rb_hash_update_by(h, kwargs, 0);
+    return rb_class_new_instance_kw(1, &h, rb_obj_class(self), TRUE);
 }
 
 /*
