@@ -15,7 +15,7 @@ class TestIONonblock < Test::Unit::TestCase
     Socket.pair(:INET, :STREAM) {|s1, s2|
       return if flush_test(s1, s2)
     }
-    skip "nonblocking IO did not work"
+    omit "nonblocking IO did not work"
   end
 
   def flush_test(r, w)
@@ -50,4 +50,24 @@ class TestIONonblock < Test::Unit::TestCase
     assert_equal(4097, result.size)
     true
   end
+
+  def test_nonblock
+    IO.pipe {|r, w|
+      w.nonblock = false
+      assert_equal(false, w.nonblock?)
+      w.nonblock do
+        assert_equal(true, w.nonblock?)
+        w.nonblock(false) do
+          assert_equal(false, w.nonblock?)
+          w.nonblock(false) do
+            assert_equal(false, w.nonblock?)
+          end
+          assert_equal(false, w.nonblock?)
+        end
+        assert_equal(true, w.nonblock?)
+      end
+      assert_equal(false, w.nonblock?)
+    }
+  rescue NotImplementedError
+  end
 end if IO.method_defined?(:nonblock)
