@@ -1,3 +1,4 @@
+# frozen_string_literal: true
 require_relative 'helper'
 
 module Psych
@@ -20,7 +21,7 @@ module Psych
     ###
     # FIXME: Syck should also support !!set as shorthand
     def test_load_from_yaml
-      loaded = Psych.load(<<-eoyml)
+      loaded = Psych.unsafe_load(<<-eoyml)
 --- !set
 foo: bar
 bar: baz
@@ -29,11 +30,11 @@ bar: baz
     end
 
     def test_loaded_class
-      assert_instance_of(Psych::Set, Psych.load(Psych.dump(@set)))
+      assert_instance_of(Psych::Set, Psych.unsafe_load(Psych.dump(@set)))
     end
 
     def test_set_shorthand
-      loaded = Psych.load(<<-eoyml)
+      loaded = Psych.unsafe_load(<<-eoyml)
 --- !!set
 foo: bar
 bar: baz
