@@ -115,11 +115,11 @@ class TestRipper::ScannerEvents < Test::Unit::TestCase
                   [[5, 0], :on_imaginary, "5.6ri", Ripper::EXPR_END],
                  ],
                  Ripper.lex("1r\n2i\n3ri\n4.2r\n5.6ri")
-     assert_lex   [[[1, 0], :on_heredoc_beg, "<<~EOS", Ripper::EXPR_BEG],
-                   [[1, 6], :on_nl, "\n", Ripper::EXPR_BEG],
-                   [[2, 0], :on_ignored_sp, "  ", Ripper::EXPR_BEG],
-                   [[2, 2], :on_tstring_content, "heredoc\n", Ripper::EXPR_BEG],
-                   [[3, 0], :on_heredoc_end, "EOS", Ripper::EXPR_BEG]
+    assert_lex   [[[1, 0], :on_heredoc_beg, "<<~EOS", Ripper::EXPR_BEG],
+                  [[1, 6], :on_nl, "\n", Ripper::EXPR_BEG],
+                  [[2, 0], :on_ignored_sp, "  ", Ripper::EXPR_BEG],
+                  [[2, 2], :on_tstring_content, "heredoc\n", Ripper::EXPR_BEG],
+                  [[3, 0], :on_heredoc_end, "EOS", Ripper::EXPR_BEG]
                  ],
                  Ripper.lex("<<~EOS\n  heredoc\nEOS")
     assert_lex   [[[1, 0], :on_tstring_beg, "'", Ripper::EXPR_BEG],
@@ -719,7 +719,7 @@ class TestRipper::ScannerEvents < Test::Unit::TestCase
                  scan('words_sep', '%w( w w w )')
     assert_equal [' ', "\n", ' ', ' '],
                  scan('words_sep', "%w( w\nw w )")
-    assert_equal ["\n\n", "\n ", ' ', ' '],
+    assert_equal ["\n", "\n", "\n", ' ', ' ', ' '],
                  scan('words_sep', "%w(\n\nw\n w w )")
   end
 
@@ -999,13 +999,21 @@ class TestRipper::ScannerEvents < Test::Unit::TestCase
     assert_equal [:on_parse_error, "Invalid escape character syntax", "\\C-\u{3042}"], err
   end
 
-  def test_invalid_hex_escape
+  def test_invalid_hex_escape_string
     err = nil
-    assert_equal ['U'], scan('tstring_content', '"\\xU"') {|*e| err = e}
-    assert_equal [:on_parse_error, "invalid hex escape", "\\x"], err
-
-    err = nil
-    assert_equal ['U'], scan('tstring_content', '/\\xU/') {|*e| err = e}
+    assert_equal ['\\x', 'U'], scan('tstring_content', '"\\xU"') {|*e| err = e}
     assert_equal [:on_parse_error, "invalid hex escape", "\\x"], err
   end
+
+  def test_invalid_hex_escape_regexp
+    err = nil
+    assert_equal ['\\x', 'U'], scan('tstring_content', '/\\xU/') {|*e| err = e}
+    assert_equal [:on_parse_error, "invalid hex escape", "\\x"], err
+  end
+
+  def test_error_token
+    src = "{a:,aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa\n""hello}"
+    err = scan('parse_error', src) {|*e| break e}
+    assert_equal "", err[2]
+  end
 end if ripper_test
