@@ -57,7 +57,7 @@ class TestGemExtConfigureBuilder < Gem::TestCase
 
     assert_match(/^current directory:/, output.shift)
     assert_equal "#{sh_prefix_configure}#{@dest_path}", output.shift
-    assert_match %r{#{shell_error_msg}}, output.shift
+    assert_match(/#{shell_error_msg}/, output.shift)
     assert_equal true, output.empty?
   end
 
